@@ -32,45 +32,25 @@ use {
     },
 };
 
+#[derive(Default)]
 pub struct CrdsGossip {
     pub crds: Crds,
-    pub id: Pubkey,
-    pub shred_version: u16,
     pub push: CrdsGossipPush,
     pub pull: CrdsGossipPull,
 }
 
-impl Default for CrdsGossip {
-    fn default() -> Self {
-        CrdsGossip {
-            crds: Crds::default(),
-            id: Pubkey::default(),
-            shred_version: 0,
-            push: CrdsGossipPush::default(),
-            pull: CrdsGossipPull::default(),
-        }
-    }
-}
-
 impl CrdsGossip {
-    pub fn set_self(&mut self, id: &Pubkey) {
-        self.id = *id;
-    }
-    pub fn set_shred_version(&mut self, shred_version: u16) {
-        self.shred_version = shred_version;
-    }
-
     /// process a push message to the network
-    /// Returns origins' pubkeys of upserted values.
+    /// Returns unique origins' pubkeys of upserted values.
     pub fn process_push_message(
         &mut self,
         from: &Pubkey,
         values: Vec<CrdsValue>,
         now: u64,
-    ) -> Vec<Pubkey> {
+    ) -> HashSet<Pubkey> {
         values
             .into_iter()
-            .flat_map(|val| {
+            .filter_map(|val| {
                 let origin = val.pubkey();
                 self.push
                     .process_push_message(&mut self.crds, from, val, now)
@@ -83,18 +63,18 @@ impl CrdsGossip {
     /// remove redundant paths in the network
     pub fn prune_received_cache<I>(
         &mut self,
+        self_pubkey: &Pubkey,
         origins: I, // Unique pubkeys of crds values' owners.
         stakes: &HashMap<Pubkey, u64>,
     ) -> HashMap</*gossip peer:*/ Pubkey, /*origins:*/ Vec<Pubkey>>
     where
         I: IntoIterator<Item = Pubkey>,
     {
-        let self_pubkey = self.id;
         origins
             .into_iter()
             .flat_map(|origin| {
                 self.push
-                    .prune_received_cache(&self_pubkey, &origin, stakes)
+                    .prune_received_cache(self_pubkey, &origin, stakes)
                     .into_iter()
                     .zip(std::iter::repeat(origin))
             })
@@ -106,8 +86,9 @@ impl CrdsGossip {
         pending_push_messages: Vec<CrdsValue>,
         now: u64,
     ) -> HashMap<Pubkey, Vec<CrdsValue>> {
-        let self_pubkey = self.id;
-        self.process_push_message(&self_pubkey, pending_push_messages, now);
+        for entry in pending_push_messages {
+            let _ = self.crds.insert(entry, now);
+        }
         self.push.new_push_messages(&self.crds, now)
     }
 
@@ -161,21 +142,24 @@ impl CrdsGossip {
         } else {
             offset
         };
-        let entries = chunks
-            .enumerate()
-            .map(|(k, chunk)| {
-                let index = (offset + k as DuplicateShredIndex) % MAX_DUPLICATE_SHREDS;
-                let data = CrdsData::DuplicateShred(index, chunk);
-                CrdsValue::new_signed(data, keypair)
-            })
-            .collect();
-        self.process_push_message(&pubkey, entries, timestamp());
+        let entries = chunks.enumerate().map(|(k, chunk)| {
+            let index = (offset + k as DuplicateShredIndex) % MAX_DUPLICATE_SHREDS;
+            let data = CrdsData::DuplicateShred(index, chunk);
+            CrdsValue::new_signed(data, keypair)
+        });
+        let now = timestamp();
+        for entry in entries {
+            if let Err(err) = self.crds.insert(entry, now) {
+                error!("push_duplicate_shred faild: {:?}", err);
+            }
+        }
         Ok(())
     }
 
     /// add the `from` to the peer's filter of nodes
     pub fn process_prune_msg(
         &self,
+        self_pubkey: &Pubkey,
         peer: &Pubkey,
         destination: &Pubkey,
         origin: &[Pubkey],
@@ -186,8 +170,8 @@ impl CrdsGossip {
         if expired {
             return Err(CrdsGossipError::PruneMessageTimeout);
         }
-        if self.id == *destination {
-            self.push.process_prune_msg(&self.id, peer, origin);
+        if self_pubkey == destination {
+            self.push.process_prune_msg(self_pubkey, peer, origin);
             Ok(())
         } else {
             Err(CrdsGossipError::BadPruneDestination)
@@ -198,6 +182,8 @@ impl CrdsGossip {
     /// * ratio - number of actives to rotate
     pub fn refresh_push_active_set(
         &mut self,
+        self_pubkey: &Pubkey,
+        self_shred_version: u16,
         stakes: &HashMap<Pubkey, u64>,
         gossip_validators: Option<&HashSet<Pubkey>>,
     ) {
@@ -205,18 +191,20 @@ impl CrdsGossip {
             &self.crds,
             stakes,
             gossip_validators,
-            &self.id,
-            self.shred_version,
+            self_pubkey,
+            self_shred_version,
             self.crds.num_nodes(),
             CRDS_GOSSIP_NUM_ACTIVE,
         )
     }
 
     /// generate a random request
+    #[allow(clippy::too_many_arguments)]
     pub fn new_pull_request(
         &self,
         thread_pool: &ThreadPool,
         self_keypair: &Keypair,
+        self_shred_version: u16,
         now: u64,
         gossip_validators: Option<&HashSet<Pubkey>>,
         stakes: &HashMap<Pubkey, u64>,
@@ -228,7 +216,7 @@ impl CrdsGossip {
             thread_pool,
             &self.crds,
             self_keypair,
-            self.shred_version,
+            self_shred_version,
             now,
             gossip_validators,
             stakes,
@@ -250,8 +238,7 @@ impl CrdsGossip {
     where
         I: IntoIterator<Item = CrdsValue>,
     {
-        self.pull
-            .process_pull_requests(&mut self.crds, callers, now);
+        CrdsGossipPull::process_pull_requests(&mut self.crds, callers, now);
     }
 
     pub fn generate_pull_responses(
@@ -260,8 +247,7 @@ impl CrdsGossip {
         output_size_limit: usize, // Limit number of crds values returned.
         now: u64,
     ) -> Vec<Vec<CrdsValue>> {
-        self.pull
-            .generate_pull_responses(&self.crds, filters, output_size_limit, now)
+        CrdsGossipPull::generate_pull_responses(&self.crds, filters, output_size_limit, now)
     }
 
     pub fn filter_pull_responses(
@@ -302,14 +288,16 @@ impl CrdsGossip {
 
     pub fn make_timeouts(
         &self,
+        self_pubkey: Pubkey,
         stakes: &HashMap<Pubkey, u64>,
         epoch_duration: Duration,
     ) -> HashMap<Pubkey, u64> {
-        self.pull.make_timeouts(self.id, stakes, epoch_duration)
+        self.pull.make_timeouts(self_pubkey, stakes, epoch_duration)
     }
 
     pub fn purge(
         &mut self,
+        self_pubkey: &Pubkey,
         thread_pool: &ThreadPool,
         now: u64,
         timeouts: &HashMap<Pubkey, u64>,
@@ -321,11 +309,9 @@ impl CrdsGossip {
         }
         if now > self.pull.crds_timeout {
             //sanity check
-            assert_eq!(timeouts[&self.id], std::u64::MAX);
+            assert_eq!(timeouts[self_pubkey], std::u64::MAX);
             assert!(timeouts.contains_key(&Pubkey::default()));
-            rv = self
-                .pull
-                .purge_active(thread_pool, &mut self.crds, now, timeouts);
+            rv = CrdsGossipPull::purge_active(thread_pool, &mut self.crds, now, timeouts);
         }
         self.crds
             .trim_purged(now.saturating_sub(5 * self.pull.crds_timeout));
@@ -339,7 +325,6 @@ impl CrdsGossip {
             crds: self.crds.clone(),
             push: self.push.mock_clone(),
             pull: self.pull.mock_clone(),
-            ..*self
         }
     }
 }
@@ -371,11 +356,8 @@ mod test {
 
     #[test]
     fn test_prune_errors() {
-        let mut crds_gossip = CrdsGossip {
-            id: Pubkey::new(&[0; 32]),
-            ..CrdsGossip::default()
-        };
-        let id = crds_gossip.id;
+        let mut crds_gossip = CrdsGossip::default();
+        let id = Pubkey::new(&[0; 32]);
         let ci = ContactInfo::new_localhost(&Pubkey::new(&[1; 32]), 0);
         let prune_pubkey = Pubkey::new(&[2; 32]);
         crds_gossip
@@ -385,10 +367,16 @@ mod test {
                 0,
             )
             .unwrap();
-        crds_gossip.refresh_push_active_set(&HashMap::new(), None);
+        crds_gossip.refresh_push_active_set(
+            &id,
+            0,               // shred version
+            &HashMap::new(), // stakes
+            None,            // gossip validators
+        );
         let now = timestamp();
         //incorrect dest
         let mut res = crds_gossip.process_prune_msg(
+            &id,
             &ci.id,
             &Pubkey::new(hash(&[1; 32]).as_ref()),
             &[prune_pubkey],
@@ -397,11 +385,25 @@ mod test {
         );
         assert_eq!(res.err(), Some(CrdsGossipError::BadPruneDestination));
         //correct dest
-        res = crds_gossip.process_prune_msg(&ci.id, &id, &[prune_pubkey], now, now);
+        res = crds_gossip.process_prune_msg(
+            &id,             // self_pubkey
+            &ci.id,          // peer
+            &id,             // destination
+            &[prune_pubkey], // origins
+            now,
+            now,
+        );
         res.unwrap();
         //test timeout
         let timeout = now + crds_gossip.push.prune_timeout * 2;
-        res = crds_gossip.process_prune_msg(&ci.id, &id, &[prune_pubkey], now, timeout);
+        res = crds_gossip.process_prune_msg(
+            &id,             // self_pubkey
+            &ci.id,          // peer
+            &id,             // destination
+            &[prune_pubkey], // origins
+            now,
+            timeout,
+        );
         assert_eq!(res.err(), Some(CrdsGossipError::PruneMessageTimeout));
     }
 }
