@@ -18,13 +18,11 @@ use {
         crds_gossip_error::CrdsGossipError,
         crds_value::CrdsValue,
         ping_pong::PingCache,
+        weighted_shuffle::WeightedShuffle,
     },
     itertools::Itertools,
     lru::LruCache,
-    rand::{
-        distributions::{Distribution, WeightedIndex},
-        Rng,
-    },
+    rand::Rng,
     rayon::{prelude::*, ThreadPool},
     solana_runtime::bloom::{AtomicBloom, Bloom},
     solana_sdk::{
@@ -77,7 +75,8 @@ impl solana_sdk::sanitize::Sanitize for CrdsFilter {
 }
 
 impl CrdsFilter {
-    pub fn new_rand(num_items: usize, max_bytes: usize) -> Self {
+    #[cfg(test)]
+    pub(crate) fn new_rand(num_items: usize, max_bytes: usize) -> Self {
         let max_bits = (max_bytes * 8) as f64;
         let max_items = Self::max_items(max_bits, FALSE_RATE, KEYS);
         let mask_bits = Self::mask_bits(num_items as f64, max_items as f64);
@@ -210,7 +209,7 @@ impl Default for CrdsGossipPull {
 impl CrdsGossipPull {
     /// generate a random request
     #[allow(clippy::too_many_arguments)]
-    pub fn new_pull_request(
+    pub(crate) fn new_pull_request(
         &self,
         thread_pool: &ThreadPool,
         crds: &Crds,
@@ -237,13 +236,10 @@ impl CrdsGossipPull {
         if peers.is_empty() {
             return Err(CrdsGossipError::NoPeers);
         }
-        let mut peers = {
-            let mut rng = rand::thread_rng();
-            let num_samples = peers.len() * 2;
-            let index = WeightedIndex::new(weights).unwrap();
-            let sample_peer = move || peers[index.sample(&mut rng)];
-            repeat_with(sample_peer).take(num_samples)
-        };
+        let mut rng = rand::thread_rng();
+        let mut peers = WeightedShuffle::new(&mut rng, &weights)
+            .unwrap()
+            .map(|i| peers[i]);
         let peer = {
             let mut rng = rand::thread_rng();
             let mut ping_cache = ping_cache.lock().unwrap();
@@ -275,7 +271,7 @@ impl CrdsGossipPull {
         now: u64,
         gossip_validators: Option<&HashSet<Pubkey>>,
         stakes: &HashMap<Pubkey, u64>,
-    ) -> Vec<(f32, &'a ContactInfo)> {
+    ) -> Vec<(u64, &'a ContactInfo)> {
         let mut rng = rand::thread_rng();
         let active_cutoff = now.saturating_sub(PULL_ACTIVE_TIMEOUT_MS);
         crds.get_nodes()
@@ -309,7 +305,9 @@ impl CrdsGossipPull {
                 let since = (now.saturating_sub(req_time).min(3600 * 1000) / 1024) as u32;
                 let stake = get_stake(&item.id, stakes);
                 let weight = get_weight(max_weight, since, stake);
-                (weight, item)
+                // Weights are bounded by max_weight defined above.
+                // So this type-cast should be safe.
+                ((weight * 100.0) as u64, item)
             })
             .collect()
     }
@@ -318,12 +316,12 @@ impl CrdsGossipPull {
     /// This is used for weighted random selection during `new_pull_request`
     /// It's important to use the local nodes request creation time as the weight
     /// instead of the response received time otherwise failed nodes will increase their weight.
-    pub fn mark_pull_request_creation_time(&mut self, from: Pubkey, now: u64) {
+    pub(crate) fn mark_pull_request_creation_time(&mut self, from: Pubkey, now: u64) {
         self.pull_request_time.put(from, now);
     }
 
     /// process a pull request
-    pub fn process_pull_requests<I>(&mut self, crds: &mut Crds, callers: I, now: u64)
+    pub(crate) fn process_pull_requests<I>(crds: &mut Crds, callers: I, now: u64)
     where
         I: IntoIterator<Item = CrdsValue>,
     {
@@ -335,14 +333,13 @@ impl CrdsGossipPull {
     }
 
     /// Create gossip responses to pull requests
-    pub fn generate_pull_responses(
-        &self,
+    pub(crate) fn generate_pull_responses(
         crds: &Crds,
         requests: &[(CrdsValue, CrdsFilter)],
         output_size_limit: usize, // Limit number of crds values returned.
         now: u64,
     ) -> Vec<Vec<CrdsValue>> {
-        self.filter_crds_values(crds, requests, output_size_limit, now)
+        Self::filter_crds_values(crds, requests, output_size_limit, now)
     }
 
     // Checks if responses should be inserted and
@@ -351,7 +348,7 @@ impl CrdsGossipPull {
     //  .0 => responses that update the owner timestamp
     //  .1 => responses that do not update the owner timestamp
     //  .2 => hash value of outdated values which will fail to insert.
-    pub fn filter_pull_responses(
+    pub(crate) fn filter_pull_responses(
         &self,
         crds: &Crds,
         timeouts: &HashMap<Pubkey, u64>,
@@ -397,7 +394,7 @@ impl CrdsGossipPull {
     }
 
     /// process a vec of pull responses
-    pub fn process_pull_responses(
+    pub(crate) fn process_pull_responses(
         &mut self,
         crds: &mut Crds,
         from: &Pubkey,
@@ -429,7 +426,7 @@ impl CrdsGossipPull {
             .extend(failed_inserts.into_iter().zip(std::iter::repeat(now)));
     }
 
-    pub fn purge_failed_inserts(&mut self, now: u64) {
+    pub(crate) fn purge_failed_inserts(&mut self, now: u64) {
         if FAILED_INSERTS_RETENTION_MS < now {
             let cutoff = now - FAILED_INSERTS_RETENTION_MS;
             let outdated = self
@@ -475,7 +472,6 @@ impl CrdsGossipPull {
 
     /// filter values that fail the bloom filter up to max_bytes
     fn filter_crds_values(
-        &self,
         crds: &Crds,
         filters: &[(CrdsValue, CrdsFilter)],
         mut output_size_limit: usize, // Limit number of crds values returned.
@@ -556,8 +552,7 @@ impl CrdsGossipPull {
     }
 
     /// Purge values from the crds that are older then `active_timeout`
-    pub fn purge_active(
-        &mut self,
+    pub(crate) fn purge_active(
         thread_pool: &ThreadPool,
         crds: &mut Crds,
         now: u64,
@@ -572,7 +567,7 @@ impl CrdsGossipPull {
 
     /// For legacy tests
     #[cfg(test)]
-    pub fn process_pull_response(
+    fn process_pull_response(
         &mut self,
         crds: &mut Crds,
         from: &Pubkey,
@@ -916,7 +911,7 @@ pub(crate) mod tests {
             &node_keypair.pubkey(),
             0,
         )));
-        let node = CrdsGossipPull::default();
+        let mut node = CrdsGossipPull::default();
         let mut pings = Vec::new();
         let ping_cache = Mutex::new(PingCache::new(
             Duration::from_secs(20 * 60), // ttl
@@ -954,19 +949,20 @@ pub(crate) mod tests {
             ),
             Err(CrdsGossipError::NoPeers)
         );
-        let new = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), 0);
+        let now = 1625029781069;
+        let new = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), now);
         ping_cache
             .lock()
             .unwrap()
             .mock_pong(new.id, new.gossip, Instant::now());
         let new = CrdsValue::new_unsigned(CrdsData::ContactInfo(new));
-        crds.insert(new.clone(), 0).unwrap();
+        crds.insert(new.clone(), now).unwrap();
         let req = node.new_pull_request(
             &thread_pool,
             &crds,
             &node_keypair,
             0,
-            0,
+            now,
             None,
             &HashMap::new(),
             PACKET_DATA_SIZE,
@@ -975,6 +971,27 @@ pub(crate) mod tests {
         );
         let (peer, _) = req.unwrap();
         assert_eq!(peer, *new.contact_info().unwrap());
+
+        node.mark_pull_request_creation_time(new.contact_info().unwrap().id, now);
+        let offline = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), now);
+        let offline = CrdsValue::new_unsigned(CrdsData::ContactInfo(offline));
+        crds.insert(offline, now).unwrap();
+        let req = node.new_pull_request(
+            &thread_pool,
+            &crds,
+            &node_keypair,
+            0,
+            now,
+            None,
+            &HashMap::new(),
+            PACKET_DATA_SIZE,
+            &ping_cache,
+            &mut pings,
+        );
+        // Even though the offline node should have higher weight, we shouldn't request from it
+        // until we receive a ping.
+        let (peer, _) = req.unwrap();
+        assert_eq!(peer, *new.contact_info().unwrap());
     }
 
     #[test]
@@ -1104,10 +1121,9 @@ pub(crate) mod tests {
         );
 
         let mut dest_crds = Crds::default();
-        let dest = CrdsGossipPull::default();
         let (_, filters) = req.unwrap();
         let mut filters: Vec<_> = filters.into_iter().map(|f| (caller.clone(), f)).collect();
-        let rsp = dest.generate_pull_responses(
+        let rsp = CrdsGossipPull::generate_pull_responses(
             &dest_crds,
             &filters,
             /*output_size_limit=*/ usize::MAX,
@@ -1125,7 +1141,7 @@ pub(crate) mod tests {
             .unwrap();
 
         //should skip new value since caller is to old
-        let rsp = dest.generate_pull_responses(
+        let rsp = CrdsGossipPull::generate_pull_responses(
             &dest_crds,
             &filters,
             /*output_size_limit=*/ usize::MAX,
@@ -1143,7 +1159,7 @@ pub(crate) mod tests {
                 .map(|(_, filter)| (caller.clone(), filter.clone()))
                 .collect::<Vec<_>>()
         });
-        let rsp = dest.generate_pull_responses(
+        let rsp = CrdsGossipPull::generate_pull_responses(
             &dest_crds,
             &filters,
             /*output_size_limit=*/ usize::MAX,
@@ -1192,16 +1208,15 @@ pub(crate) mod tests {
         );
 
         let mut dest_crds = Crds::default();
-        let mut dest = CrdsGossipPull::default();
         let (_, filters) = req.unwrap();
         let filters: Vec<_> = filters.into_iter().map(|f| (caller.clone(), f)).collect();
-        let rsp = dest.generate_pull_responses(
+        let rsp = CrdsGossipPull::generate_pull_responses(
             &dest_crds,
             &filters,
             /*output_size_limit=*/ usize::MAX,
             0,
         );
-        dest.process_pull_requests(
+        CrdsGossipPull::process_pull_requests(
             &mut dest_crds,
             filters.into_iter().map(|(caller, _)| caller),
             1,
@@ -1232,7 +1247,6 @@ pub(crate) mod tests {
         let new = CrdsValue::new_unsigned(CrdsData::ContactInfo(new));
         node_crds.insert(new, 0).unwrap();
 
-        let mut dest = CrdsGossipPull::default();
         let mut dest_crds = Crds::default();
         let new_id = solana_sdk::pubkey::new_rand();
         let new = ContactInfo::new_localhost(&new_id, 1);
@@ -1267,13 +1281,13 @@ pub(crate) mod tests {
             );
             let (_, filters) = req.unwrap();
             let filters: Vec<_> = filters.into_iter().map(|f| (caller.clone(), f)).collect();
-            let rsp = dest.generate_pull_responses(
+            let rsp = CrdsGossipPull::generate_pull_responses(
                 &dest_crds,
                 &filters,
                 /*output_size_limit=*/ usize::MAX,
                 0,
             );
-            dest.process_pull_requests(
+            CrdsGossipPull::process_pull_requests(
                 &mut dest_crds,
                 filters.into_iter().map(|(caller, _)| caller),
                 0,
@@ -1316,7 +1330,7 @@ pub(crate) mod tests {
         )));
         let node_label = entry.label();
         let node_pubkey = node_label.pubkey();
-        let mut node = CrdsGossipPull::default();
+        let node = CrdsGossipPull::default();
         node_crds.insert(entry, 0).unwrap();
         let old = CrdsValue::new_unsigned(CrdsData::ContactInfo(ContactInfo::new_localhost(
             &solana_sdk::pubkey::new_rand(),
@@ -1332,7 +1346,7 @@ pub(crate) mod tests {
         );
         // purge
         let timeouts = node.make_timeouts(node_pubkey, &HashMap::new(), Duration::default());
-        node.purge_active(&thread_pool, &mut node_crds, node.crds_timeout, &timeouts);
+        CrdsGossipPull::purge_active(&thread_pool, &mut node_crds, node.crds_timeout, &timeouts);
 
         //verify self is still valid after purge
         assert_eq!(
